# $Idaemons: /home/cvs/pkgtools/pkgtools.rb,v 1.35 2001/09/12 06:52:50 knu Exp $

PREFIX = "/usr/local"

begin
  require 'fnmatch' unless File.respond_to?(:fnmatch?)
rescue LoadError
  if ENV['RUBY_STATIC']
    raise "fnmatch module not found"
  else
    exec '/usr/bin/env', 'RUBY_STATIC=1', "#{PREFIX}/bin/ruby_s", $0, *ARGV
  end
end

require "pkg"
require "ports"

require "tempfile"

require "shellwords"
include Shellwords

$portsdb = PortsDB.instance.setup
$ports_dir = $portsdb.ports_dir
$pkgdb = PkgDB.instance.setup
$pkgdb_dir = $pkgdb.db_dir
$packages_base = ENV['PKGREPOSITORY'] || File.join($ports_dir, 'packages')
$packages_dir = File.join($packages_base, ENV['PKGREPOSITORYSUBDIR'] || 'All')
$tmpdir = ENV['PKG_TMPDIR'] || ENV['TMPDIR'] || '/var/tmp'

ENV['PKG_PATH'] = $packages_dir if not ENV.key?('PKG_PATH')
$pkg_path = ENV['PKG_PATH']

$verbose = false
$sudo_args = ['sudo']
$sudo = false

def parse_pattern(str, regex = false)
  if str[0] == ?:
    regex = true
    str = str[1..-1]
  end

  if regex
    Regexp.new(str)
  else
    str
  end
end

def stty_sane
  system '/bin/stty', 'sane'
end

def progress_message(message, io = STDOUT)
  io.puts "--->  " + message
end

def information_message(message, io = STDERR)
  io.puts "++ " + message
end

def warning_message(message, io = STDERR)
  io.puts "** " + message
end

def all?(str)
  /^a/i =~ str
end

def yes?(str)
  /^y/i =~ str
end

def no?(str)
  /^n/i =~ str
end

def yesno_str(yes)
  if yes then 'yes' else 'no' end
end
  
def prompt_yesno(message = "OK?", yes_by_default = false)
  print "#{message} [#{yesno_str(yes_by_default)}] "
  STDOUT.flush

  input = (STDIN.gets || '').strip

  if yes_by_default
    !no?(input)
  else
    yes?(input)
  end
end

def prompt_yesnoall(message = "OK?", yes_by_default = false)
  print "#{message} ([y]es/[n]o/[a]ll) [#{yesno_str(yes_by_default)}] "
  STDOUT.flush

  input = (STDIN.gets || '').strip

  if all?(input)
    :all
  elsif yes_by_default
    !no?(input)
  else
    yes?(input)
  end
end

def matchlen(a, b)
  i = 0
  0.upto(a.size) { |i| a[i] != b[i] and break }
  i
end

def input_line(prompt, add_history = nil, completion_proc = nil)
  prev_proc = Readline.completion_proc

  Readline.completion_append_character = nil if Readline.respond_to?(:completion_append_character=)
  Readline.completion_proc = completion_proc if completion_proc.respond_to?(:call)

  Readline.readline(prompt, add_history)
ensure
  Readline.completion_proc = prev_proc if prev_proc.respond_to?(:call)
end

def input_file(prompt, dir, add_history = nil)
  Dir.chdir_do(dir) {
    return input_line(prompt, add_history, Readline::FILENAME_COMPLETION_PROC)
  }
end

OPTIONS_NONE	= 0x00
OPTIONS_SKIP	= 0x01
OPTIONS_DELETE	= 0x02
OPTIONS_ALL	= 0x04
OPTIONS_HISTORY	= 0x08

def choose_from_options(message = 'Input?', options = nil, flags = OPTIONS_NONE)
  skip		= (flags & OPTIONS_SKIP).nonzero?
  delete	= (flags & OPTIONS_DELETE).nonzero?
  all		= (flags & OPTIONS_ALL).nonzero?
  history	= (flags & OPTIONS_HISTORY).nonzero?

  completion_proc = nil

  unless options.nil?
    case options.size
    when 0
      return :skip
    else
      completion_proc = proc { |head|
	len = head.size
	options.select { |option| head == option[0, len] }
      }
    end
  end

  loop do
    input = input_line(message + ' (? to help): ', history, completion_proc)

    if input.nil?
      print "\n"

      next if not delete

      if all
	ans = prompt_yesnoall("Delete this?", true)
      else
	ans = prompt_yesno("Delete this?", true)
      end

      if ans == :all
	return :delete_all
      elsif ans
	return :delete
      end

      next
    end

    input.strip!

    case input
    when '.'
      return :abort
    when '?'
      print ' [Enter] to skip,' if skip
      print ' [Ctrl]+[D] to delete,' if delete
      print '  [.][Enter] to abort, [Tab] to complete'
      print "\n"
      next
    when ''
      if skip
	if all
	  ans = prompt_yesnoall("Skip this?", true)
	else
	  ans = prompt_yesno("Skip this?", true)
	end

	if ans == :all
	  return :skip_all
	elsif ans
	  return :skip
	end
      end

      next
    else
      if options.include?(input)
	return input
      end

      print "Please choose one of these:\n"

      if options.size <= 20
	puts options.join('  ')
      else
	puts options[0, 20].join('  ') + "  ..."
      end
    end
  end

  # not reached
end

def shelljoin(*args)
  args.collect { |arg|
    if /[*?{}\[\]<>()~&|\\$;\'\`\"\s]/ =~ arg
      '"' + arg.gsub(/([$\\\"\`])/, "\\\\\\1") + '"'
    else
      arg
    end
  }.join(' ')
end

def sudo(*args)
  if $sudo
    if $sudo_args.grep(/%s/).empty?
      args = $sudo_args + args
    else
      args = $sudo_args.map { |arg|
	format(arg, shelljoin(*args)) rescue arg
      }
    end

    progress_message "[Executing a command as root: " + shelljoin(*args) + "]"
  end

  system(*args)
end
alias system! sudo

def script(file, *args)
  if file
    system('/usr/bin/script', '-qa', file, *args)
  else
    system(*args)
  end
end

def script!(file, *args)
  if file
    system!('/usr/bin/script', '-qa', file, *args)
  else
    system!(*args)
  end
end

def grep_file(re, file)
  system '/usr/bin/egrep', '-q', re.source, file
end

def update_pkgdep(oldpkgname, newpkgname)
  return if oldpkgname == newpkgname

  progress_message "Updating dependency info" if $verbose

  # gsub's below are unnecessary in reality as pkgname shouldn't contain \'s
  required_by_re_oldpkgname = /^#{Regexp.quote(oldpkgname)}$/
  required_by_sub_newpkgname = newpkgname.gsub(/\\/, "\\\\")

  contents_re_oldpkgname = /^(@pkgdep +)#{Regexp.quote(oldpkgname)}$/
  contents_sub_newpkgname = "\\1" + newpkgname.gsub(/\\/, "\\\\")

  $pkgdb.glob('*', false) do |pkgname|
    subst_file(required_by_re_oldpkgname,
	       required_by_sub_newpkgname,
	       $pkgdb.pkg_required_by(pkgname))

    subst_file(contents_re_oldpkgname,
	       contents_sub_newpkgname,
	       $pkgdb.pkg_contents(pkgname))
  end
end

def subst_file(pattern, subst, file, backup_file = nil)
  changed_lines = []

  File.open(file) do |r|
    lines = r.collect { |line|
      newline = line.gsub(pattern, subst)
      if newline != line
	changed_lines << [line, newline]
      end
      newline
    }

    r.close

    unless changed_lines.empty?
      w = Tempfile.new(File.basename(file))
      w.print lines
      w.close

      progress_message "Modifying #{file}" if $verbose

      File.chmod 0644, w.path

      if File.writable? file
	system '/bin/mv', '-f', file, backup_file if backup_file
	system '/bin/cp', '-f', w.path, file
      else
	system! '/bin/mv', '-f', file, backup_file if backup_file
	system! '/bin/cp', '-f', w.path, file
      end
    end
  end

  changed_lines
rescue
  nil
end

def search_paths(command)
  ENV['PATH'].split(':').each do |dir|
    path = File.join(dir, command)
    stat = File.stat(path)
    return path if stat.file? && stat..executable?(path)
  end

  nil
end

def show_results(results, done_service = 'done')
  if $verbose
    if results.empty?
      warning_message "No package has been #{done_service}."
      return 0
    end

    progress_message "Reporting the results (+:succeeded / -:skipped / !:failed)"
  else
    results.find { |item, e, | !e.nil? } or return 0

    warning_message "The following packages were not #{done_service} (-:skipped / !:failed)"
  end

  write_results(results)
end

def write_results(results, io = STDOUT, prefix = "\t", verbose = $verbose)
  errors = 0

  results.each do |item, e, info|
    if info
      item = "#{item} (#{info})"
    end

    case e
    when nil	# succeded
      io.puts "#{prefix}+ #{item}" if verbose
    when false	# skipped
      io.puts "#{prefix}- #{item}"
    when Exception
      io.puts "#{prefix}! #{item}\t(#{e.message})"
      errors += 1
    else
      io.puts "#{prefix}! #{item}\t(#{e})"
      errors += 1
    end
  end

  errors
end

class Dir
  def self.chdir_do(dir, &block)
    pwd = Dir.pwd

    begin
      Dir.chdir(dir)

      block.call
    ensure
      Dir.chdir(pwd)
    end
  end
end

module Enumerable
  def partition
    a = []
    b = []
    each do |x|
      if yield(x)
	a << x
      else
	b << x
      end
    end

    return a, b
  end
end
